local __exports = LibStub:NewLibrary("ovale/states/AzeriteArmor", 90102)
if not __exports then return end
local __class = LibStub:GetLibrary("tslib").newClass
local aceEvent = LibStub:GetLibrary("AceEvent-3.0", true)
local wipe = wipe
local pairs = pairs
local tostring = tostring
local ipairs = ipairs
local sort = table.sort
local insert = table.insert
local concat = table.concat
local C_Item = C_Item
local ItemLocation = ItemLocation
local C_AzeriteEmpoweredItem = C_AzeriteEmpoweredItem
local GetSpellInfo = GetSpellInfo
local __enginecondition = LibStub:GetLibrary("ovale/engine/condition")
local returnBoolean = __enginecondition.returnBoolean
local returnConstant = __enginecondition.returnConstant
local azeriteSlots = {
    [1] = true,
    [3] = true,
    [5] = true
}
__exports.OvaleAzeriteArmor = __class(nil, {
    constructor = function(self, equipments, ovale, ovaleDebug)
        self.equipments = equipments
        self.traits = {}
        self.output = {}
        self.debugOptions = {
            azeraittraits = {
                name = "Azerite traits",
                type = "group",
                args = {
                    azeraittraits = {
                        name = "Azerite traits",
                        type = "input",
                        multiline = 25,
                        width = "full",
                        get = function(info)
                            return self:debugTraits()
                        end
                    }
                }
            }
        }
        self.handleInitialize = function()
            self.module:RegisterMessage("Ovale_EquipmentChanged", self.itemChanged)
            self.module:RegisterEvent("AZERITE_EMPOWERED_ITEM_SELECTION_UPDATED", self.handleAzeriteEmpoweredItemSelectionUpdated)
            self.module:RegisterEvent("PLAYER_ENTERING_WORLD", self.handlePlayerEnteringWorld)
        end
        self.handleDisable = function()
            self.module:UnregisterMessage("Ovale_EquipmentChanged")
            self.module:UnregisterEvent("AZERITE_EMPOWERED_ITEM_SELECTION_UPDATED")
            self.module:UnregisterEvent("PLAYER_ENTERING_WORLD")
        end
        self.itemChanged = function()
            local slotId = self.equipments.lastChangedSlot
            if slotId ~= nil and azeriteSlots[slotId] then
                self:updateTraits()
            end
        end
        self.handleAzeriteEmpoweredItemSelectionUpdated = function()
            self:updateTraits()
        end
        self.handlePlayerEnteringWorld = function()
            self:updateTraits()
        end
        self.azeriteTraitRank = function(positionalParams, namedParams, atTime)
            local spellId = positionalParams[1]
            local value = self:traitRank(spellId)
            return returnConstant(value)
        end
        self.hasAzeriteTrait = function(positionalParams, namedParams, atTime)
            local spellId = positionalParams[1]
            local value = self:hasTrait(spellId)
            return returnBoolean(value)
        end
        self.module = ovale:createModule("OvaleAzeriteArmor", self.handleInitialize, self.handleDisable, aceEvent)
        for k, v in pairs(self.debugOptions) do
            ovaleDebug.defaultOptions.args[k] = v
        end
    end,
    registerConditions = function(self, ovaleCondition)
        ovaleCondition:registerCondition("hasazeritetrait", false, self.hasAzeriteTrait)
        ovaleCondition:registerCondition("azeritetraitrank", false, self.azeriteTraitRank)
    end,
    updateTraits = function(self)
        self.traits = {}
        for slotId in pairs(azeriteSlots) do
            local itemSlot = ItemLocation:CreateFromEquipmentSlot(slotId)
            if C_Item.DoesItemExist(itemSlot) and C_AzeriteEmpoweredItem.IsAzeriteEmpoweredItem(itemSlot) then
                local allTraits = C_AzeriteEmpoweredItem.GetAllTierInfo(itemSlot)
                for _, traitsInRow in pairs(allTraits) do
                    for _, powerId in pairs(traitsInRow.azeritePowerIDs) do
                        local isEnabled = C_AzeriteEmpoweredItem.IsPowerSelected(itemSlot, powerId)
                        if isEnabled then
                            local powerInfo = C_AzeriteEmpoweredItem.GetPowerInfo(powerId)
                            local name = GetSpellInfo(powerInfo.spellID)
                            if self.traits[powerInfo.spellID] then
                                local rank = self.traits[powerInfo.spellID].rank
                                self.traits[powerInfo.spellID].rank = rank + 1
                            else
                                self.traits[powerInfo.spellID] = {
                                    spellID = powerInfo.spellID,
                                    name = name,
                                    rank = 1
                                }
                            end
                            break
                        end
                    end
                end
            end
        end
    end,
    hasTrait = function(self, spellId)
        return (self.traits[spellId] and true) or false
    end,
    traitRank = function(self, spellId)
        if  not self.traits[spellId] then
            return 0
        end
        return self.traits[spellId].rank
    end,
    debugTraits = function(self)
        wipe(self.output)
        local array = {}
        for k, v in pairs(self.traits) do
            insert(array, tostring(v.name) .. ": " .. tostring(k) .. " (" .. v.rank .. ")")
        end
        sort(array)
        for _, v in ipairs(array) do
            self.output[#self.output + 1] = v
        end
        return concat(self.output, "\n")
    end,
})
